#include <Arduino.h>
#include <Wire.h>
#include "ButtonBoard.h"

/* Arduino pins */
const uint8_t bb_pin_ready_init = A3;
const uint8_t bb_i2c_addr = 0x34;

ButtonBoard btnboard( bb_pin_ready_init, bb_i2c_addr );

void setup()
{
  err rc;
  
  Wire.begin();
  Serial.begin( 115200 );
  Serial.print( "\nButtonBoard Demo\n" );
  
  uint8_t version;
  rc = btnboard.ReadVersion( &version );
  Serial.print( "Version " );
  Serial.print( version );
  Serial.print( ", rc " );
  Serial.print( rc );
  Serial.println();
  
  ButtonBoard::E_BTN_SWITCH_MODE switch_modes[BB_BTN_COUNT];
  switch_modes[0] = btnboard.BTN_SWITCH_MODE_HOLD;
  switch_modes[1] = btnboard.BTN_SWITCH_MODE_CAPTURE;
  rc = btnboard.WriteSwitchMode( switch_modes, 2 );
  Serial.print( "Write Switch Mode, rc " );
  Serial.print( rc );
  Serial.println();
  
  ButtonBoard::E_BTN_SWITCH_MULTI_MODE switch_multi_modes[BB_BTN_COUNT];
  switch_multi_modes[0] = btnboard.BTN_MULTI_MODE_MATCH;
  switch_multi_modes[1] = btnboard.BTN_MULTI_MODE_THRESHOLD;
  rc = btnboard.WriteSwitchMultiMode( switch_multi_modes, 2 );
  Serial.print( "Write Switch Multi Mode, rc " );
  Serial.print( rc );
  Serial.println();
  
  ButtonBoard::E_BTN_SWITCH_STATE_SET switch_state_set[BB_BTN_COUNT];
  switch_state_set[0] = btnboard.BTN_SWITCH_STATE_SET_OFF;
  switch_state_set[1] = btnboard.BTN_SWITCH_STATE_SET_OFF;
  rc = btnboard.WriteSwitchStateSet( switch_state_set, 2 );
  Serial.print( "Write Switch State Set, rc " );
  Serial.print( rc );
  Serial.println();
  
  uint8_t switch_multi_count[BB_BTN_COUNT];
  switch_multi_count[0] = 2;
  switch_multi_count[1] = 3;
  rc = btnboard.WriteSwitchMultiCount( switch_multi_count, 2 );
  Serial.print( "Write Switch Multi Count, rc " );
  Serial.print( rc );
  Serial.println();
  
  ButtonBoard::E_PORTS_INPUT ports_input[BB_BTN_COUNT];
  ports_input[0] = btnboard.PORTS_INPUT_INVERTED;
  ports_input[1] = btnboard.PORTS_INPUT_ON;
  rc = btnboard.WritePortsInput( ports_input, 2 );
  Serial.print( "Write Ports Input, rc " );
  Serial.print( rc );
  Serial.println();
  
  ButtonBoard::E_PORTS_OUTPUT ports_output[BB_BTN_COUNT];
  ports_output[0] = btnboard.PORTS_OUTPUT_ON;
  ports_output[1] = btnboard.PORTS_OUTPUT_CH0;
  rc = btnboard.WritePortsOutput( ports_output, 2 );
  Serial.print( "Write Ports Output, rc " );
  Serial.print( rc );
  Serial.println();
  
  ButtonBoard::E_PIN_PULLS pin_pulls[BB_BTN_COUNT];
  pin_pulls[0] = btnboard.PIN_PULLS_STRONG_PULL_UP;
  pin_pulls[1] = btnboard.PIN_PULLS_TRISTATE;
  rc = btnboard.WritePinPulls( pin_pulls, 2 );
  Serial.print( "Write Pin Pulls, rc " );
  Serial.print( rc );
  Serial.println();
  
  rc = btnboard.WriteLedOutput( btnboard.LED_OUTPUT_CH1 );
  Serial.print( "Write LED, rc " );
  Serial.print( rc );
  Serial.println();
  
  uint8_t debounce_ms[BB_BTN_COUNT];
  debounce_ms[0] = 55;
  debounce_ms[1] = 110;
  rc = btnboard.WriteDebounceMs( debounce_ms, 2 );
  Serial.print( "Write Debounce Ms, rc " );
  Serial.print( rc );
  Serial.println();
  
  uint16_t multi_timeout_ms[BB_BTN_COUNT];
  multi_timeout_ms[0] = 333;
  multi_timeout_ms[1] = 666;
  rc = btnboard.WriteMultiTimeoutMs( multi_timeout_ms, 2 );
  Serial.print( "Write Debounce Ms, rc " );
  Serial.print( rc );
  Serial.println();
  
  uint16_t hold_timeout_ms[BB_BTN_COUNT];
  hold_timeout_ms[0] = 55;
  hold_timeout_ms[1] = 110;
  rc = btnboard.WriteHoldTimeoutMs( hold_timeout_ms, 2 );
  Serial.print( "Write Timeout Ms, rc " );
  Serial.print( rc );
  Serial.println();
  
  rc = btnboard.WriteCmd( btnboard.CMD_APPLY_CONFIG );
  Serial.print( "Write Cmd, rc " );
  Serial.print( rc );
  Serial.println();
  
  ButtonBoard::port_values_t port_values[BB_BTN_COUNT];
  rc = btnboard.ReadPorts( port_values );
  Serial.print( "Port Values " );
  Serial.print( port_values[0].pin_inputs, HEX );
  Serial.print( " " );
  Serial.print( port_values[0].btn_inputs, HEX );
  Serial.print( " " );
  Serial.print( port_values[0].switch_outputs, HEX );
  Serial.print( " " );
  Serial.print( port_values[0].pin_outputs, HEX );
  Serial.print( "; " );
  Serial.print( port_values[1].pin_inputs, HEX );
  Serial.print( " " );
  Serial.print( port_values[1].btn_inputs, HEX );
  Serial.print( " " );
  Serial.print( port_values[1].switch_outputs, HEX );
  Serial.print( " " );
  Serial.print( port_values[1].pin_outputs, HEX );
  Serial.print( ", rc " );
  Serial.print( rc );
  Serial.println();
  
  ButtonBoard::E_UART_CONFIG uart_config;
  uart_config = btnboard.UART_CONFIG_MASK_TEXT | btnboard.UART_CONFIG_MASK_HEX;
  rc = btnboard.WriteUartConfig( uart_config );
  Serial.print( "Write UART Config, rc " );
  Serial.print( rc );
  Serial.println();
  
  ButtonBoard::addr_t addr;
  addr.addr_link_open = 22;
  addr.addr_link_short = 33;
  addr.addr_override = 7;
  rc = btnboard.WriteAddr( &addr );
  Serial.print( "Write Addr, rc " );
  Serial.print( rc );
  Serial.println();
  
  uint8_t addr_active;
  rc = btnboard.ReadAddrActive( &addr_active );
  Serial.print( "I2C Addr Active " );
  Serial.print( addr_active );
  Serial.print( ", rc " );
  Serial.print( rc );
  Serial.println();
}

byte val = 0;
char str[20];

void loop() {
  uint8_t i;
  err rc;
  btn_events_t events;
  
  while ( !btnboard.Ready() );
  
  rc = btnboard.ReadEvents( &events );
  
  if ( rc != 0 )
    Serial.print( "Error\n" );
  else
  {
    for ( i=0; i<(BB_BTN_COUNT<<1); i++ )
    {
      sprintf( str, "%02hX %hu ", events[i].event, events[i].presses );
      Serial.print( str );
    }
    Serial.println();
  }
  
  delay(100);
}
